@@ -4,10 +4,9 @@
 #include <bpf/bpf_helpers.h>
 #include <bpf/bpf_endian.h>
 
-#include "xdp_dispatcher_v1.h"
+#include "xdp_dispatcher.h"
 
 #define XDP_METADATA_SECTION "xdp_metadata"
-#define XDP_DISPATCHER_VERSION_V1 1
 #define XDP_DISPATCHER_RETVAL 31
 
 
