@@ -32,6 +32,20 @@ DEFINE_SAMPLE_INIT(xdp_redirect_devmap);
 const struct devmap_opts defaults_redirect_devmap = { .mode = XDP_MODE_NATIVE,
 						      .interval = 2 };
 
+
+static struct bpf_program *egress_prog(struct xdp_redirect_devmap *skel,
+				       enum devmap_egress_action action)
+{
+	switch (action) {
+	case DEVMAP_EGRESS_DROP:
+		return skel->progs.xdp_redirect_devmap_egress_drop;
+	case DEVMAP_EGRESS_NONE:
+	case DEVMAP_EGRESS_FORWARD:
+	default:
+		return skel->progs.xdp_redirect_devmap_egress;
+	}
+}
+
 int do_redirect_devmap(const void *cfg, __unused const char *pin_root_path)
 {
 	const struct devmap_opts *opt = cfg;
@@ -59,6 +73,11 @@ int do_redirect_devmap(const void *cfg, __unused const char *pin_root_path)
 	if (opt->stats)
 		mask |= SAMPLE_REDIRECT_CNT;
 
+	if (!opt->load_egress && opt->egress_action != DEVMAP_EGRESS_NONE) {
+		pr_warn("egress-action option was set without load-egress\n");
+		goto end;
+	}
+
 restart:
 	skel = xdp_redirect_devmap__open();
 	if (!skel) {
@@ -76,7 +95,7 @@ restart:
 
 	if (tried) {
 		tx_port_map = skel->maps.tx_port_general;
-		bpf_program__set_autoload(skel->progs.xdp_redirect_devmap_egress, false);
+		bpf_program__set_autoload(egress_prog(skel, opt->egress_action), false);
 #ifdef HAVE_LIBBPF_BPF_MAP__SET_AUTOCREATE
 		bpf_map__set_autocreate(skel->maps.tx_port_native, false);
 #else
@@ -184,8 +203,12 @@ restart:
 	}
 
 	devmap_val.ifindex = opt->iface_out.ifindex;
-	if (opt->load_egress)
-		devmap_val.bpf_prog.fd = bpf_program__fd(skel->progs.xdp_redirect_devmap_egress);
+	if (opt->load_egress) {
+		struct bpf_program *prog = egress_prog(skel, opt->egress_action);
+
+		devmap_val.bpf_prog.fd = bpf_program__fd(prog);
+	}
+
 	ret = bpf_map_update_elem(bpf_map__fd(tx_port_map), &key, &devmap_val, 0);
 	if (ret < 0) {
 		pr_warn("Failed to update devmap value: %s\n",
